#!/usr/bin/perl
#
# AntiVir Milter for sendmail
# Installation script.
#
# Version: 1.6


# just to avoid errors
use strict;

# important globals
my $system=`uname`;
chomp($system);

my $av_user = "root";
my $av_group = "antivir";
my $vdffile = "antivir.vdf";
my $conffile = "avmilter.conf";
my $scanconffile = "antivir.conf";
my $scanfile = "avmilter.scan";
my $ignorefile = "avmilter.ignore";
my $warnfile = "avmilter.warn";
my $enginefile = "antivir";
my $SuSE8xscript = "rc.avmilter.SuSE8x";
my $SuSE8xversion ="";
my $SuSEscript = "rc.avmilter.SuSE";
my $RedHatscript = "rc.avmilter.redhat";
my $defaultscript = "rc.avmilter";
my $solarisscript= "rc.avmilter.solaris";
my $openbsdscript= "rc.avmilter.openbsd";
my $freebsdscript= "rc.avmilter.freebsd";
my $linkfile = "rcavmilter";
my $sbindir = "/usr/sbin";
my $proddir = ".";
my $progdir = "/usr/lib/AntiVir";
my $progdir_default = $progdir;
my $initdir = "/etc/rc.d:/etc/init.d:/sbin/init.d";
my $initscript = "avmilter";
my $confdir = "/etc";
my $license = "";
my $keyfile1 = "avmgate.key";
my $keyfile2 = "hbedv.key";
my $keyfile3 = "";
my $enginecallname = "sendmail";
my $prodnamelong = "AntiVir Milter";
my $prodnameshort = "avmilter";
my $milter_file = "avmilter";
my $milter_file_bin = "avmilter.bin";
my $prodname = "AntiVir";
my $supportmail = "support\@antivir.de";
my $versionbanner = "AvMilter";
my $licensefile = "LICENSE";
my $guardconffile = "avguard.conf";
my $do_update = 0;
my $rc_yorn = "n";
my $versionbanner = "product version:";
my $prod_version_installed = "";
my $prod_version_pkg = "";
my $updaterscript = "avupdater";
my $rcscript = "$proddir/script/rcscript";
my $usrbin = "/usr/bin";
my $show_notes = 0;

my $libc=`$proddir/script/getsysteminfo --classification`;
chomp($libc);

######################
# Our simple Main ;-)
######################
sub main  {

  # Check if current user is root
  if ($> != 0) {
    print "\nYou must be root, to execute this script.\n\n";
    exit 1;
  }

  # Confirm EULA
  eula();

# Stop when running on libc5
  if($libc eq "linux_libc5"){
   die ("\n$prodnamelong does not run on a $libc system.\n\n");
  }

  # Shutdown running processes
  check_progs();

# Create group
  if($system eq "FreeBSD") {
   system("/usr/sbin/pw groupadd $av_group > /dev/null 2>&1");
  }else{
   system("/usr/sbin/groupadd $av_group > /dev/null 2>&1");
  }

  # ask for other directories (mandir,bindir,confdir,progdir)
  ask_dirs();

  # install updater
  install_updater();

  # copy files
  copy_files();

  # check for guard and it's IncludePath's and ExcludePath's
  check_guard();

  # display INSTALL 
  mta_readme();
}

#####################################
# copy the files to the destinations
#####################################
sub copy_files {

  my $rc_file = "";
  my $dir = $ENV{_};
  my $version = "";
  $dir =~ /(.*?)\/install.pl$/;
  $dir = "." unless $dir = $1;

# copy system dependent binaries
	if(system("cp -f $proddir/bin/$libc/$milter_file $proddir/bin/ > /dev/null 2>&1") != 0) {
	 print "\nCould copy executable ($proddir/bin/$libc/$milter_file) for this system!\n";
	 die;
	}

	if(system("cp -f $proddir/bin/$libc/$enginefile $proddir/bin/ > /dev/null 2>&1") !=0) {
	 print "\nCould copy executable ($proddir/bin/$libc/$enginefile) for this system!\n";
	 die;
	}


  #################################
  # 1) scan engine, vdf and milter
  #################################
  # search for existing scan engine, and backup if they differ
  if (-e "$progdir/$enginefile") {

     # compare old scan engine with new scan engine if they have different versions
	  if (comp_v(`$dir/$proddir/bin/$enginefile --version |head -n1`,
					  `$progdir/$enginefile --version |head -n1`) == 1)
		{
			  install ("f", $av_user, $av_group, 0755, "$progdir/$enginefile", "$dir/$proddir/bin/$enginefile");
			  chomp ($version = `$progdir/$enginefile --version |head -n1`);
			  print "\nUpdating $progdir/$enginefile to version: $version";
     } else {

       print "\nSkipping $enginefile update";
     }

     # install the new VDF if it's newer than the old one
     if (comp_v(`head -n4 $dir/$proddir/vdf/$vdffile | tr -d '\015' |grep Version: | sed -e 's/Version: //'`,
                         `head -n4 $progdir/$vdffile | tr -d '\015' |grep Version: | sed -e 's/Version: //'`) == 1) {
       install ("f", $av_user, $av_group, 0644, "$progdir/$vdffile", "$dir/$proddir/vdf/$vdffile");
       chomp ($version = `head -n4 $dir/$proddir/vdf/$vdffile | tr -d '\015' |grep Version: | sed -e 's/Version: //'`);
       print "\nUpdating $progdir/$vdffile with version: $version";
     } else {

       print "\nSkipping $vdffile update";
     }
  } else {

    # And install scan engine and VDF
    install ("f", $av_user, $av_group, 0755, "$progdir/$enginefile", "$dir/$proddir/bin/$enginefile");
    chomp ($version = `$progdir/$enginefile --version |head -n1`);
    print "\nInstalling new $progdir/$enginefile with version: $version";

    install ("f", $av_user, $av_group, 0644, "$progdir/$vdffile", "$dir/$proddir/vdf/$vdffile");
    chomp ($version = `head -n4 $progdir/$vdffile | tr -d '\015' |grep Version: | sed -e 's/Version: //'`);
    print "\nInstalling new $progdir/$vdffile with version: $version";

  }

  # Install milter
  if((-e "$sbindir/$milter_file") && (!-l "$sbindir/$milter_file"))
  {
   $show_notes=1;
  }

  if(-e "$progdir/$milter_file_bin")
  {
   $prod_version_installed = `$progdir/$milter_file_bin -V | grep -i '$versionbanner' | sed 's/$versionbanner//' | tr -d '\040'`;
  }

  $prod_version_pkg = `$dir/$proddir/bin/$prodnameshort -V | grep -i '$versionbanner' | sed 's/$versionbanner//' | tr -d '\040'`;
 
  if(comp_v($prod_version_pkg, $prod_version_installed) == 1)
  {
   print "\n\nInstalling new $milter_file binary to $progdir/$milter_file_bin";
   install ("f", $av_user, $av_group, 0755, "$progdir/$milter_file_bin", "$dir/$proddir/bin/$milter_file");
   print "\nCreating link from $progdir/$milter_file_bin to $sbindir/$milter_file";
   system("ln -s $progdir/$milter_file_bin $sbindir/$milter_file > /dev/null 2>&1");
  }

  ##################
  # 2) config files
  ##################
  # if there are no installed config files...
  if (!-e "$confdir/$conffile") {

    install ("f", $av_user, $av_group, 0664, "$confdir/$conffile", "$dir/$proddir/etc/$conffile");
    print "\n\nInstalling config file $confdir/$conffile";
  } else {

    print "\n\nFound existing $confdir/$conffile. Skipping.";
  }

  if (!-e "$confdir/$scanconffile") {

    install ("f", $av_user, $av_group, 0664, "$confdir/$scanconffile", "$dir/$proddir/etc/$scanconffile");
    print "\n\nInstalling config file $confdir/$scanconffile";
  } else {

    print "\nFound existing $confdir/$scanconffile. Skipping.";
  }

  if (!-e "$confdir/$scanfile") {

    install ("f", $av_user, $av_group, 0664, "$confdir/$scanfile", "$dir/$proddir/etc/$scanfile");
    print "\n\nInstalling config file $confdir/$scanfile";
  } else {

    print "\nFound existing $confdir/$scanfile. Skipping.";
  }

  if (!-e "$confdir/$ignorefile") {

    install ("f", $av_user, $av_group, 0664, "$confdir/$ignorefile", "$dir/$proddir/etc/$ignorefile");
    print "\n\nInstalling config file $confdir/$ignorefile";
  } else {

    print "\nFound existing $confdir/$ignorefile. Skipping.";
  }

  if (!-e "$confdir/$warnfile") {

    install ("f", $av_user, $av_group, 0664, "$confdir/$warnfile", "$dir/$proddir/etc/$warnfile");
    print "\n\nInstalling config file $confdir/$warnfile";
  } else {

    print "\nFound existing $confdir/$warnfile. Skipping.";
  }

  #######################
  # 3) spool directories
  #######################
  if ($do_update == 0) {

    if (!-d "/var/spool/$prodnameshort/incoming") {
      install ("d", "uucp", $av_group, 0700, "/var/spool/$prodnameshort/incoming");
      print "\n\nCreating /var/spool/$prodnameshort/incoming";
    }
    if (!-d "/var/spool/$prodnameshort/outgoing") {
      install ("d", "uucp", $av_group, 0700, "/var/spool/$prodnameshort/outgoing");
      print "\nCreating /var/spool/$prodnameshort/outgoing";
    }
    if (!-d "/var/spool/$prodnameshort/rejected") {
      install ("d", "uucp", $av_group, 0700, "/var/spool/$prodnameshort/rejected");
      print "\nCreating /var/spool/$prodnameshort/rejected\n";
    }
  }

  #######################
  # 4) start/stop script
  #######################
	if (-e "/etc/SuSE-release") 
	{
	 $SuSE8xversion=`grep VERSION /etc/SuSE-release | sed s/"VERSION = "/""/ | cut -d . -f1`;

	 chop($SuSE8xversion);

	 if ($SuSE8xversion >= "8") 
	 {
	  system("cp -f $dir/$proddir/init/$SuSE8xscript $progdir/$initscript");
	  print "\n\nInstalling start/stop script for a SuSE System\n";
	 }
	 else
	 {
	  system("cp -f $dir/$proddir/init/$SuSEscript $progdir/$initscript");
	  print "\n\nInstalling start/stop script for a SuSE System\n";
	 }

	} elsif (-e "/etc/redhat-release") {
		system("cp -f $dir/$proddir/init/$RedHatscript $progdir/$initscript");
		print "\n\nInstalling start/stop script for a Redhat System\n";

	} elsif ($system eq "OpenBSD") {
	 	system("cp -f $dir/$proddir/init/$openbsdscript $progdir/$initscript");
		print "\n\nInstalling start/stop script for OpenBSD\n";

	} elsif ($system eq "FreeBSD") {
	 	system("cp -f $dir/$proddir/init/$freebsdscript $progdir/$initscript");
		print "\n\nInstalling start/stop script for FreeBSD\n";

	} elsif ($system eq "SunOS") {
	 	system("cp -f $dir/$proddir/init/$solarisscript $progdir/$initscript");
		print "\n\nInstalling start/stop script for Solaris\n";

	} else {
		system("cp -f $dir/$proddir/init/$defaultscript $progdir/$initscript");
		print "\n\nInstalling default start/stop script"; 
	}

	system("ln -sf $progdir/$initscript $sbindir/$linkfile > /dev/null 2>&1");
	system("ln -s $progdir/$enginefile $usrbin/$enginefile > /dev/null 2>&1");

	system("chmod 755 $progdir/$initscript");
	system("chown $av_user:$av_group $progdir/$initscript");

	my $autostart="";
	print "\nWould you like AvMilter to start automatically? [n] ";
	chomp($autostart = <STDIN>);

	if ($autostart =~ /(y|yes|j|ja)$/i) { 
	 system("/bin/sh $rcscript install $progdir/$initscript");
	}


  ######################################### 
  # 5) if there is a license file, copy it
  ######################################### 
  # Change access rights for license key
  if ($do_update == 0) {  

    if ( -e "$progdir/$keyfile1" && -f "$progdir/$keyfile1") {
        chmod (0644,"$progdir/$keyfile1")
    }
    if ( -e "$progdir/$keyfile2" && -f "$progdir/$keyfile2") {
        chmod (0644,"$progdir/$keyfile2")
    }
    if ( -e "$progdir/$keyfile3" && -f "$progdir/$keyfile3") {
        chmod (0644,"$progdir/$keyfile3")
    }
    if ($license ne "") {
      print "\n\nInstalling license file to $progdir/$keyfile1\n";
      install ("f", $av_user, $av_group, 0644, "$progdir/$keyfile1", $license);
    }
  }
}

##############################
# Ask for working directories
##############################
sub ask_dirs {

  # 1) Get directory for binary
#  $sbindir = ask("Enter the path where $prodnamelong binary will be located",$sbindir);

 if(-e $progdir && !-d $progdir) {
  die ("\"$progdir\" is not a directory");
 }

 if(!-e $progdir) {
  mkdir $progdir,0755 or die ("can not create directory \"$progdir\"");
 }

  # 3) Ask for license
  if (!-e "$progdir/$keyfile1" && !-e "$progdir/$keyfile2" && !-e "progdir/$keyfile3") {
    $license = get_license();
  }
}

################################
# ask user for existing license
################################
sub get_license {

  my $file = "";
  my $ok = "";

  while ($ok eq "") {

    print "\nEnter the path to your license file []: ";
    chomp ($file = <STDIN>);

    # test path
    if ($file eq "") {

      #print "      $prodnamelong will run in non-key mode.\n",
      #      "      To get a license key please read the FAQ or send a mail to $supportmail\n\n";

      $ok = 1;
    } else {

      # check if file exists
      if ( -e "$file") {

          return $file;

      } else {
        print "      The file $file doesn't exists. Try again!",
      }
    }
  }

  return "";
}

###########################################
# Ask a question and return the answer
# 
# If it's a non existing directory, 
# the user will be asked wheater to create
# it or not.
###########################################
sub ask {

  my $question = shift;
  my $default = shift;
  my $nodir = shift;
  my $ok = "";
  my $input = "";
  my $create = "";

  while ($ok eq "") {
    print "\n$question \n(default is: $default): ";
    chomp ($input = <STDIN>);
    
    # if default was chosen
    if ($input eq "") {
      $input = $default;
      $ok = 1;
    }

    # if parameter is a directory => check if is exists
    if (!defined($nodir)) {
      # try if directory exists
      if ( !-d "$input") {
        print "      The directory $input doesn't exists. Try again!\n";
        print "      Shall I create it? [y] ";
        chomp ($create = <STDIN>);

        if ( $create =~ /^(j|y)/ || ($create eq "")){
	 system("mkdir $input > /dev/null 2>&1");
	 system("chmod 755 $input > /dev/null 2>&1");
	 $ok = 1;
	}
      } else {
      
        $ok = 1;
      }
    } else { 

      $ok = 1;
    }
  } 
  return $input;
}

################
# Show the EULA
################
sub eula {

  my $pager=$ENV{PAGER};
  my $ok = "";

  $pager="more" if(!$pager);

  system("clear && $pager $proddir/$licensefile");

  do {
    print "\nDo you agree the LICENSE [n] ";
    chomp($ok = <STDIN>);

  } while ($ok !~ /(y|yes|n|no|j|ja|nein)$/i);

  if ($ok !~ /(y|yes|j|ja)$/i) {
    exit 2;
  } else {
    system("clear");
  }
}



###########################
# Install automatic updater
###########################
sub install_updater {

  my $ok = "";
  print "\nAn automatic internet updater of AntiVir for UNIX is available.\n";
  print "This is a daemon that will run in the background\n";
  print "and automatically check for updates (internet access is required).\n";
  print "You may also manually check for updates using:\n" ;
  print "\n";
  print "		$enginefile --update\n";
  print "\n";
  print "You do not need to install the automatic internet updater in order\n";
  print "to manually check for updates. Please read the README file for more\n";
  print "information on updates and how they can best suit you.\n";
  print "\nInstall the automatic internet updater? [n] ";
  chomp($ok = <STDIN>);

	if (($ok =~ /(y|yes|ja)$/i)){
	 	print "\nInstalling updater script to $progdir\n";
	 	system("cp -f $proddir/script/$updaterscript $progdir");

		print "\nWould you like the automatic updater to start automatically? [y] ";
		chomp($ok=<STDIN>);

		if (($ok =~ /(y|yes|ja)$/i) || ($ok eq "")){
		 system("/bin/sh $rcscript install $progdir/$updaterscript");
		}
	}
}


########################
# Show the INSTALL file
########################
sub mta_readme {

 if($show_notes)
 {
  print "\n\n************************************************************************\n";
  print "Please note that the program binary has been moved to $progdir!\n";
  print "From now on, the binary (avmilter.bin) will be installed to $progdir\n";
  print "and not to $sbindir. You should replace the binary in $sbindir with\n";
  print "a link to the correspondig file in $progdir.\n";
  print "************************************************************************\n";
 }

 print "\n\n$prodnamelong is now installed.\n\n"; 
 print "*NOTE* Please adapt your sendmail.cf:\n\n";
 print "Include the following lines in the configuration file sendmail.cf of sendmail:\n";
 print "X$prodnameshort, S=inet:3333\@localhost, F=R, T=S:10m;R:10m;E:10m\n";
 print "O InputMailFilters=$prodnameshort\n\n";
 print "You can also include the following line to your sendmail.mc file, if you want\n";
 print "to create your own sendmail.cf file.\n";
 print "INPUT_MAIL_FILTER(`$prodnameshort',`S=inet:3333\@localhost,F=R,T=S:10m;R:10m;E:10m')\n";

 print "\nYou can either start $prodnameshort by running $progdir/$prodnameshort.bin or by\n";
 print "using the rc-script \"$progdir/$prodnameshort start\".\n";
}

################################
# compares 2 version numbers
# eg. (6.12.2.1, 6.12.2.3) => 2
################################
sub comp_v {

  # returns 0 if both version's are the same
  # returns 1 if first version is newer than the second version
  # returns 2 if second version is newer than the first version

  my @version1 = split (/[-\.]/,shift);
  my @version2 = split (/[-\.]/,shift);

  for (0..$#version1) {
    return 2 if ($version1[$_] < $version2[$_]);
    return 1 if ($version1[$_] > $version2[$_]);
  }
  return 0;
}

################################
# Install's a file or directory
################################
sub install {
                                        ##### PARAMETERs #################
  my $f_d = shift;                      # (f)ile or (d)irectory
  my $user = getpwnam(shift);           # Owner of file or directory
  my $group = getgrnam(shift);          # Group of file or directory
  my $mask = shift;                     # Rights of file or directory
  my $dest = shift;                     # Destination of file or directory
  my $src = shift if ($f_d eq "f");     # Source file only when $f_d="f"

  my $dir = "";
  my @subs;

  # installing a file
  if ($f_d eq "f") {

    # if file is already existing => backup it
    if (-e $dest) {

      rename($dest,"$dest.saved_by_install");
      system ("cp $src $dest");
      chown ($user,$group,$dest);
      chmod ($mask,$dest);

    # or create new one with directories
    } else {

      reverse ($dest) =~ /(.*?)\/(.*?)$/;
      my $filename = reverse ($1);
      my $path = reverse ($2);
      @subs = split(/\//,$path);

      # create directories
      foreach (@subs) {

        $dir .= "$_/";

        if (!-d $dir) {
          mkdir ($dir,0755);
          chown ($user,$group,$dir);
        }
      }

      # copy source file to the destination
      system ("cp $src $dest");
      chmod ($mask,$dest);
      chown ($user,$group,$dest);
    }

  # installing a directory
  } elsif ($f_d eq "d") {

    @subs = split(/\//,$dest);

    foreach (@subs) {

      $dir .= "$_/";

      if (!-d $dir) {
        mkdir ($dir,$mask);
        chown ($user,$group,$dir);
      }
    }
  }
}

############################
# Check for AVGuard Paths
# and display an ERROR when
# SpoolPath is not excluded
# in /etc/avguard.conf
############################
sub check_guard {

  my $include;
  my $exclude;
  my ($tmp,$spool,$in,$ex);

  # Read SpoolDir from configfile, otherwise set SpoolDir to /var/spool/
  $spool = "/var/spool/$proddir" unless $spool = readConfig("$confdir/$conffile","SpoolDir");

  # check if TemporaryDir exists in confifile
  if (!($tmp = readConfig("$confdir/$conffile","TemporaryDir"))) {

    # otherwise check if /var/tmp exists
    if (-x "/var/tmp") {
      $tmp = "/var/tmp";

    # if not, our TemporaryDir is /tmp
    } else {
      $tmp = "/tmp";
    }
  }

  # get IncludeDir's from /etc/avguard.conf, exit if not exists
  $include = readConfig("$confdir/$guardconffile","IncludePath");

  # get ExcludeDir's from /etc/avguard.conf
  $exclude = readConfig("$confdir/$guardconffile","ExcludePath");


  # Search in all IncludeDir's
  foreach $in (split /:/,$include) {
   
    if ($spool =~ /^($in)/) {

      my $ok = 0;

      foreach $ex (split /:/,$exclude) {
        $ok = 1 if ($spool =~ /^$ex/); 
      }

      print "

########################################################################################
#
# WARNING:
# ========
#
# The IncludePath ($1) from /etc/$guardconffile includes the
# SpoolDirectory ($spool) used by $prodnamelong !!!
#
# Please exclude it manualy (eg. ExcludePath $spool) in /etc/$guardconffile,
# or change SpoolDirectory in /etc/$conffile,
# otherwise $prodnamelong gets no Viruses ;-(
#
#######################################################################################\n\n" if !$ok;
    }




    if ($tmp =~ /^($in)/) {

      my $ok = 0;

      foreach $ex (split /:/,$exclude) {
        $ok = 1 if ($tmp =~ /^$ex/);
      }

      print "

#######################################################################################
#
# WARNING:
# ========
#
# The IncludePath ($1) from /etc/$guardconffile includes the
# TemporaryDir ($tmp) used by $prodnamelong !!!
#
# Please exclude it manualy (eg. ExcludePath $tmp) in /etc/$guardconffile,
# or change TemporaryDir in /etc/$conffile,
# otherwise $prodnamelong gets no Viruses ;-(
#
#######################################################################################\n\n" if !$ok;
    }
  }
}


# usage: readConfig("filename",[Parameter])
# returns: if [Parameter] is defined the value is passed back,
#          otherwise a hash with all parameters is passed back
sub readConfig {

  my $filename = shift;
  my $param = shift;
  my %config;

  if(open(CONF,$filename)) {

    while(<CONF>) {

      chomp;
      next if /\w+\#|\#|^$/;
      my ($name,$value) = /(\w+)[\ \t]+(.*?)$/;

      if ($config{$name}) {
        $config{$name} .= ":$value";
      } else {
        $config{$name} = $value;
      }
    }
  }

  return $config{$param} if $param;
  return %config if keys %config > 0;
  return undef;
}

#################################
# Notify if processes are running
#################################
sub check_progs{

  my $avmilter="avmilter";
# The processes are called with --mailgate even for milter
  my $antivir_mailgate="antivir.*--mailgate";
  my $pid;

  if ($system eq "SunOS")
  {
   if($pid=`ps -Af -o "pid" -o "comm" | grep -i "$avmilter" | grep -v grep`)
   {
    chomp($pid);
    $pid =~ s/^\s+//;
    print "AntiVir Milter ($pid) already running, please stop the processes and start the installation again.\n";
    exit 4;
   }

   if($pid=`ps -Af -o "pid" -o "comm" | grep -i "$antivir_mailgate" | grep -v grep`)
   {
    chomp($pid);
    $pid =~ s/^\s+//;
    print "AntiVir Milter ($pid) already running, please stop the processes and start the installation again.\n";
    exit 4;
   }
  }
  else
  {
   # older ps commands dont know these options
   if($pid=`ps ax | grep "$avmilter" | grep -v grep`)
   {
    chomp($pid);
    $pid =~ s/^\s+//;
    print "AntiVir Milter ($pid) already running, please stop the processes and start the installation again.\n";
    exit 4;
   }

   if($pid=`ps ax | grep "$antivir_mailgate" | grep -v grep`)
   {
    chomp($pid);
    $pid =~ s/^\s+//;
    print "AntiVir Milter ($pid) already running, please stop the processes and start the installation again.\n";
    exit 4;
   }
  }
}


#################################
# Check if scan daemon is running
#################################
sub check_scan_daemon {

	my $daemon="";

	if($system eq "SunOS") {
 	$daemon = `ps -Af | grep $enginefile | grep -v grep`;
	}
	else {
	$daemon = `ps awx | grep $enginefile | grep -v grep`;
	}	

	if ($daemon ne "") {
		print "\nAnother scan daemon of $enginefile is running. Please\n";
		print "stop $prodnamelong and start the installation\n";
		print "again. If you are using the Guard of $prodname\n";
		print "Workstation/Server or the internet updater\n";
		print "stop the scan daemon, too.\n";
		print "If you have done this and this message appears\n";
		print "again kill the $enginefile process manually!\n";
		exit 2;
	}  

}

main();
